#!/usr/bin/env python
import sys
import struct
import argparse


# https://github.com/ceph/ceph/blob/master/src/tools/rbd/Utils.h#L35

RBD_DIFF_BANNER = "rbd diff v1\n";
RBD_DEFAULT_SPARSE_SIZE = 4096;
RBD_IMAGE_BANNER_V2 = "rbd image v2\n"
RBD_IMAGE_DIFFS_BANNER_V2 = "rbd image diffs v2\n"
RBD_DIFF_BANNER_V2 = "rbd diff v2\n"


RBD_DIFF_FROM_SNAP  = 'f'
RBD_DIFF_TO_SNAP    = 't'
RBD_DIFF_IMAGE_SIZE = 's'
RBD_DIFF_WRITE      = 'w'
RBD_DIFF_ZERO       = 'z'
RBD_DIFF_END        = 'e'

RBD_SNAP_PROTECTION_STATUS     = 'p'

RBD_EXPORT_IMAGE_ORDER      = 'O'
RBD_EXPORT_IMAGE_FEATURES   = 'T'
RBD_EXPORT_IMAGE_STRIPE_UNIT    = 'U'
RBD_EXPORT_IMAGE_STRIPE_COUNT   = 'C'
RBD_EXPORT_IMAGE_META       = 'M'
RBD_EXPORT_IMAGE_END        = 'E'


def parse(f):
    banner = f.readline()
    if banner == RBD_DIFF_BANNER:
        export_format = 1
    else:
        export_format = 2
    print("banner: %s" % (banner.strip() ,))
    print("export_format: %s" % (export_format,))

    while True:
        tag = f.read(1)
        if tag == "":
            print("eof")
            return

        elif tag == RBD_DIFF_FROM_SNAP:
            if export_format == 2:
                f.read(4)
            l = struct.unpack("i", f.read(4))[0]
            fromsnapname = f.read(l)
            print("from: %s" % (fromsnapname,))

        elif tag == RBD_DIFF_TO_SNAP:
            if export_format == 2:
                f.read(4)
            l = struct.unpack("i", f.read(4))[0]
            endsnapname = f.read(l)
            print("snap: %s" % (endsnapname,))

        elif tag == RBD_SNAP_PROTECTION_STATUS and export_format == 2:
            f.read(4)
            f.read(8)

        elif tag == RBD_DIFF_IMAGE_SIZE:
            if export_format == 2:
                f.read(4)
            size = struct.unpack("Q", f.read(8))[0]
            print("size: %s" % (size,))

        elif tag == RBD_DIFF_WRITE:
            if export_format == 2:
                f.read(4)
            offset = struct.unpack("Q", f.read(8))[0]
            length = struct.unpack("Q", f.read(8))[0]
            print("data: offset: %s, length: %s" % (offset, length))
            f.read(length)

        elif tag == RBD_DIFF_ZERO:
            if export_format == 2:
                f.read(4)
            offset = struct.unpack("Q", f.read(8))[0]
            length = struct.unpack("Q", f.read(8))[0]
            print("zero: offset: %s, length: %s" % (offset, length))

        elif tag == RBD_DIFF_END:
            print("end")

        else:
            print("Error: unknown tag: %s" % (tag,))
            sys.exit(1)




if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("file", help="filename or - for stdout")
    args = parser.parse_args()

    if args.file == "-":
        parse(sys.stdin)
    else:
        parse(open(args.file, "rb"))
